(* Authors: Kundan Guha, Hannah Pierce-Hoffman *)


module StringMap = Map.Make(String)

type action = Netlist | LLVM_IR | Compile

let _ =
  let action = ref Compile in
  let set_action a () = action := a in
  let speclist = [
    ("-a", Arg.Unit (set_action Netlist), "Print the SAST");
    ("-l", Arg.Unit (set_action LLVM_IR), "Print the generated LLVM IR");
    ("-c", Arg.Unit (set_action Compile),
      "Check and print the generated LLVM IR (default)");
  ] in  
  let usage_msg = "usage: ./logisimple.native [-a|-l|-c] [file.sim]" in
  let channel = ref stdin in
  Arg.parse speclist (fun filename -> channel := Unix.open_process_in ("m4 " ^ filename)) usage_msg;
  let lexbuf = Lexing.from_channel !channel in
  (* Scan and then parse *)
  let ast = Parser.program Scanner.token lexbuf in
  let the_nl = Semant.check ast in
  let m = Codegen.translate the_nl in
  Llvm_analysis.assert_valid_module m;
  print_string (Llvm.string_of_llmodule m)

    
(*  print_string (Ast.string_of_gate_def ast)
  match !action with
    Netlist -> print_string (AST.string_of_gate_def ast)
  | LLVM_IR -> print_string "oops"
  (* Feed the scanned, parsed "ast" into the codegen
  | Compile -> let m = Codegen.translate ast in
    Llvm_analysis.assert_valid_module m;
    print_string (Llvm.string_of_llmodule m)*)*)
